// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "Entity.hpp"
#include "scene/Scene.hpp"
#include "assert/assert.hpp"

using namespace Dawn;

void Entity::onInit() {
  auto world = this->world.lock();
  assertNotNull(world, "World is no longer active?");

  listeners.push_back(getScene()->onUnpausedUpdate.listen([this](float delta) {
    if(this->stepTime <= 0.0f) return;
    this->stepTime = 0;
    this->getItem()->setLocalPosition(glm::vec3(this->tileX, this->tileY, this->tileZ));
  }));
}

void Entity::onDispose() {
  
}

void Entity::move(const enum EntityDirection direction) {
  entitytileposition_t newTileX, newTileY, newTileZ;
  newTileX = tileX;
  newTileY = tileY;
  newTileZ = tileZ;

  switch(direction) {
    case EntityDirection::Up:
      newTileZ--;
      break;
    case EntityDirection::Down:
      newTileZ++;
      break;
    case EntityDirection::Left:
      newTileX--;
      break;
    case EntityDirection::Right:
      newTileX++;
      break;
    default:
      assertUnreachable("Invalid direction: %d", direction);
  }

  // Get the tile at the destination, check for height if we are on stairs, etc.
  // If the tile is not walkable, return early.

  // Move the entity to the new tile.
  this->tileX = newTileX;
  this->tileY = newTileY;
  this->tileZ = newTileZ;
  this->direction = direction;
  this->stepTime = 1.0f;
}