// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PlayerPrefab.hpp"
#include "display/mesh/QuadMesh.hpp"

using namespace Dawn;

struct PlayerPrefab Dawn::createPlayerPrefab(
  const std::shared_ptr<World> world
) {
  struct PlayerPrefab player;
  player.item = world->getScene()->createSceneItem();
  player.item->setParent(world->getItem());

  player.player = player.item->addComponent<Player>();
  player.player->world = world;

  player.mesh = std::make_shared<Mesh>();
  player.mesh->createBuffers(QUAD_VERTICE_COUNT, QUAD_INDICE_COUNT);
  QuadMesh::buffer(
    player.mesh,
    glm::vec4(0, 0, 1, 1),
    glm::vec4(0, 0, 1, 1),
    0, 0
  );

  player.meshRenderer = player.item->addComponent<MeshRenderer>();
  player.meshRenderer->mesh = player.mesh;

  player.material = player.item->addComponent<SimpleTexturedMaterial>();
  player.material->setColor(COLOR_MAGENTA);

  return player;
}